#!/usr/bin/env python2

from webmarkupcontainer import WebMarkupContainer
from markupfromfile import MarkupFromFile

class Page(WebMarkupContainer, MarkupFromFile):
    """
    A top-level page. Knows how to render() its component hierarchy into an lxml.etree.Element tree
    """
    def render(self, context):
        tree = self.markupFromFile()
        self.renderInto(tree, context)
        return tree



#!/usr/bin/env python2
from component import Component

class Label(Component):
    """
    A plain text value. Suitable for use with <span>, <h[1-6]> etc.
    """
    def __init__(self, value):
        if not isinstance(value, basestring): raise Exception("Label value must be a string")
        self.value = value
    def renderInto(self, tag, context):
        tag.text = self.value



from component import Component

class WebMarkupContainer(Component):
    """
    In Wicket, this class exists primarily to be a non-abstract Component
    that can be used in page hierarchies; I haven't yet encountered any behaviour
    of its own, so it's questionable whether this class should even be included
    in smoff at this time.
    """
    pass
    


from lxml import etree
from StringIO import StringIO
import pkgutil

def basicMarkupFromClass(cls):
    """
    Load the file associated with the specific class, and return the raw element tree without processing any tags
    """
    markup = pkgutil.get_data(cls.__module__, "%s.html" % cls.__name__)
    parser = etree.HTMLParser()
    return etree.parse(StringIO(markup), parser)

def markupFromClass(cls):
    tree = basicMarkupFromClass(cls)
    #If a component's markup has a <smoff:extend> tag that means it extends its parent's markup rather than replacing it
    #TODO: Proper use of namespaces
    extendElements = tree.findall('//extend', namespaces={'smoff': "http://optim.al/smoff"})
    if(len(extendElements) > 1):
        raise Exception("Found multiple <smoff:extend> tags in the same file")
    elif(len(extendElements) == 1):
        extendElement = extendElements[0]
        parentMarkup = markupFromClass(cls.__bases__[0]) #TODO: Allow multiple inheritance
        #Extend markup is inserted into the parent markup at the <smoff:child> tag
        childElement = parentMarkup.find('//child')
        childElement.getparent().replace(childElement, extendElement)
        return parentMarkup
    else:
        return tree

class MarkupFromFile(object):
    """
    Mixin for classes that expect to have an html file with them
    (principally Page and Panel)
    """
    def markupFromFile(self):
        return markupFromClass(self.__class__)



#!/usr/bin/env python2

from distutils.core import setup

from distutils.core import Command
from unittest import TextTestRunner, TestLoader
from glob import glob
from os.path import splitext, basename, join as pjoin, walk
import os

class TestCommand(Command):
    user_options = [ ]

    def initialize_options(self):
        self._dir = os.getcwd()

    def finalize_options(self):
        pass

    def run(self):
        '''
        Finds all the tests modules in tests/, and runs them.
        '''
        testfiles = [ ]
        for t in glob(pjoin(self._dir, 'tests', '*.py')):
            if not t.endswith('__init__.py'):
                testfiles.append('.'.join(
                    ['tests', splitext(basename(t))[0]])
                )

        tests = TestLoader().loadTestsFromNames(testfiles)
        t = TextTestRunner(verbosity = 1)
        t.run(tests)

setup(name='smoff', version='0.0.6', description='Wicket-inspired template engine for python/django', author='Michael Donaghy', author_email='michael@optimalsocial.com', url='https://bitbucket.org/lmm/smoff',
 packages=['smoff', 'smoff.smdjango'],
 requires=['lxml', 'django'],
 cmdclass = { 'test': TestCommand,}
 
 )



#!/usr/bin/env python2
from smoff import SMOFF_NAMESPACE
from lxml import etree
from StringIO import StringIO
import pkgutil

class Component(object):
    """
    Base of the smoff class hierarchy
    The key concept is the hierarchy; components are arranged in a tree, corresponding to the tree in markup
    Visibility could plausibly be separated into a separate class if we wanted to keep everything ultra-decoupled
    """
    def __init__(self):
        #_children is private, intended to be accessed only via add(...)
        self._children = {}
        #visible is a public property
        self.visible = True
        
    def add(self, markupid, child):
        """
        Add child (a Component) with the given markupId (string).
        Implies that this component's markup contains a child node with smoff:id= this markupid
        into which that component should be rendered
        """
        if(markupid in self._children):
            raise Exception("Already have a child with markup id %s" % markupid)
        self._children[markupid] = child
        child.parent = self
        
    def renderInto(self, tag, context):
        """
        Render this component into the given tag (an lxml.etree.Element or compatible).
        context is a bit of a hack but I can't see a better way to do it
        """
        if not self.visible:
            #Currently we don't check anything or render subelements in the case of an invisible Component.
            #This may be worth reconsidering
            tag.attrib['style'] = 'display:none'
        else:
            #Keep track of how deep into the tree we are
            #We are only responsible for rendering our immediate children
            #We could theoretically achieve better performance by pulling the iteration up so that we only
            #traverse the tree once for a page (rather than each Component traversing its own tree during rendering)
            #but performance has not yet become a concern
            path = []
            for event, element in etree.iterwalk(tag, ('start', 'end')):
                if 'smoff:id' in element.attrib and not tag == element:
                    if 'start' == event:
                        path.append(element)
                    elif 'end' == event:
                        if(path.pop() != element):
                            #I believe this error is impossible, so throwing a non-catchable exception
                            raise Exception("Expect same element on ascent as on descent")
                        if(not path):
                            childid = element.attrib['smoff:id']
                            #Raise an exception if a component was added in markup but not in code
                            #This indicates programming error rather than runtime failure, so should
                            #not be catchable
                            self._children.pop(childid).renderInto(element, context)
            #Raise an exception if a component was added in code but not in markup
            #This indicates programming error rather than runtime failure, so should
            #not be catchable
            if(self._children):
                raise Exception("""Unable to find markup ids for %s. Did you forget to set smoff:id in your markup?
                    Markup was: %s""" % (self._children, etree.tostring(tag)))



from django.views.generic.base import View
from django.http import HttpResponse
from django.core.context_processors import csrf

from lxml import etree

class SmoffView(View):
    """
    Adapts a Page as a django class-based view.
    """
    page_class = None
    def get(self, *args, **kwargs):
        context = csrf(self.request)
        self.page_class.request = self.request
        ret = HttpResponse(etree.tostring(self.page_class().render(context)))
        #FIXME: handle multiple tabs etc.
        if 'post_handler' in context:
            self.request.session['post_handler'] = context['post_handler']
        return ret
    def post(self, *args, **kwargs):
        return self.request.session.pop('post_handler').handlePost(self.request)
